/*! 
    \file resource_manager.h
    \brief The Resource Manager checks if a file/resource is loaded.

    If a file/resource was already loaded the resourceManager will 
    return a void pointer to the desired resource.
    Otherwise it will instruct the coresponding resource-loader to load,
    and receive a pointer to it.
*/

#ifndef _RESOURCE_MANAGER_H
#define _RESOURCE_MANAGER_H

#include "base_object.h"

// FORWARD DEFINITION \\
//template<class T> class tList;
#include "list.h"                //! \todo do this by forward definition (ask Patrick)
enum ResourceType {OBJ, PRIM, WAV, MP3, OGG, IMAGE};

struct Resource
{
  void* pointer;             //!< Pointer to the Resource.
  
  char* name;                //!< Name of the Resource.
  ResourceType type;         //!< ResourceType of this Resource.
  int count;                 //!< How many times this Resource has been loaded.
};


//! The ResourceManager is a class, that decides if a file/resource should be loaded
/**
   If a file/resource was already loaded the resourceManager will 
   return a void pointer to the desired resource.
   Otherwise it will instruct the corresponding resource-loader to load,
   and receive the pointer to it.

   It does it by looking, if a desired file has already been loaded.
*/
class ResourceManager : public BaseObject 
{
 public:
  static ResourceManager* getInstance();
  virtual ~ResourceManager();

  static bool setDataDir(char* dataDir);
  static bool addImageDir(char* imageDir);
  static void* load(const char* fileName);
  static void* load(const char* fileName, ResourceType type);
  static bool unload(void* pointer);

 private:
  ResourceManager();
  static ResourceManager* singletonRef;

  static tList<Resource>* resourceList;
  static char* dataDir;
  static tList<char>* imageDirs;

  static Resource* locateResourceByName(const char* fileName);
  static Resource* locateResourceByPointer(const void* pointer);
  

  static bool isDir(const char* directory);
  static bool isFile(const char* directory);  
};

#endif /* _RESOURCE_MANAGER_H */
